package reseauSocial.implementation2018;

import facebookGhost.FacebookGhostNetwork;
import facebookGhost.RelationEvent;
import facebookGhost.User;
import grapheSimple.Chemin;
import grapheSimple.GrapheSimple;
import grapheSimple.ParcoursSimple;
import grapheX.Sommet;
import reseauSocial.core.MemberInterface;
import reseauSocial.core.SocialNetworkInterface;

import java.util.Collection;
import java.util.Observable;
import java.util.Set;


public class SocialNetWorkImpl implements SocialNetworkInterface<Member>{

    GrapheSimple<Member> network = new GrapheSimple<>();
    String name;

    ParcoursSimple browser;

    FacebookGhostNetwork fg;


    public SocialNetWorkImpl(String nomDuReseau, FacebookGhostNetwork fg ) {
        name = nomDuReseau;
        browser = new ParcoursSimple(network);
        this.fg = fg;
    }

    public SocialNetWorkImpl(String nomDuReseau) {
        name = nomDuReseau;
        browser = new ParcoursSimple(network);
    }
    public boolean existRelationEntre (Member m1, Member m2) {
        return network.existeArc(m1, m2);
    }


    public Member addMember(String nom, int age, String description) {
        Member m = new Member(nom,age,description);
        network.ajouterSommet(m);
        return m;
    }

    public Member addMember(String identifiant, boolean autreReseau) {
        Member m ;
        if (fg != null){
            User user = fg.getUser(identifiant);
            m = new MemberFGAdapter(identifiant,user);
            manageRelations(m,user);
        }else
            m = addMember(identifiant,0,"echec de connexion exterieure");
        network.ajouterSommet((Member) m);
        return m;
    }



    private void manageRelations(Member origine, User user) {
        for ( User friend : user.getFriends()){
        	Member m = getMember(friend.getId());
            if (m != null) {
                relate(3, origine, m);
                relate(3, m, origine);
            }
        }

        for ( User friend : user.getFamily()){
        	Member m = getMember(friend.getId());
            if (m != null) {
                relate(2, origine, m);
                relate(2, m, origine);
            }
        }

    }

    public Member getMember(String nomMember) {
        return network.getSommet(nomMember);
    }

    public void relate(int force,  Member source, Member destination) {
        network.ajouterArc(source, destination, force);
    }



    public Collection<Member> getMembers() {
        return network.sommets();
    }




    public int distance(Member origine, Member destination) {
        Chemin chemin = browser.cheminLePlusCourt((Sommet)origine,(Sommet) destination);
        return chemin.distance();
    }

    public Set<Member> relateToRank(
    		Member origine, int rang) {
        Set<Sommet> res = browser.voisinsAuRang((Sommet) origine, rang);
        return (Set) res;
    }

/*	public Set<Member> relateToRank(MemberInterface Member, int rang) {
		ArrayList<Chemin> amis = parcours.chemins(Member);
		ArrayList<Member> relations= new ArrayList<Member>();
		ArrayList<Member> relationsPlusCourtes= new ArrayList<Member>();
		for (Chemin chemin : amis) {
			if (chemin.size() == rang)
				relations.add((Member) chemin.arrivee());
			else
			if (chemin.size()<rang)
				relationsPlusCourtes.add((Member) chemin.arrivee());
		}
		relations.removeAll(relationsPlusCourtes);
		return relations;
	}*/

    @Override
    public String toString() {
        return "SocialNetwork [name=" + name
                + ", network=" + network + " "  + "]";
    }

    public void setOtherNetwork(FacebookGhostNetwork f) {
        fg = f;
    }

    public void update(Observable arg0,Object arg1) {
        if (! (arg1 instanceof  RelationEvent))
            return ;
        RelationEvent re = (RelationEvent)arg1;
        Member m1 = getMember(re.getU1().getId() );
        Member m2 = getMember(re.getU2().getId() );
        if ( (m1 != null) && (m2 != null) )
            if (re.getNature().equals("family")){
                relate(2, m1, m2);
                relate(2, m2, m1);
            }
            else {
                relate(3, m1, m2);
                relate(3, m2, m1);
            }

    }


}
